@@ -539,8 +539,8 @@ set_item_caption (CatalogPngExporter *ce,
 	if (idata->caption_set)
 		return;
 
-	if ((ce->caption_fields & CAPTION_COMMENT) &&
-	    (idata->comment != NULL))
+	if ((ce->caption_fields & CAPTION_COMMENT) 
+	    && (idata->comment != NULL))
 		idata->caption_row[row++] = g_strdup (idata->comment);
 	if ((ce->caption_fields & CAPTION_FILE_PATH) 
 	    && (ce->caption_fields & CAPTION_FILE_NAME)) 
