@@ -369,7 +369,7 @@ comments_save_comment (const gchar *filename,
         /* write to disk. */
 	comment_file = comments_get_comment_filename (filename);
 	dest_dir = remove_level_from_path (comment_file);
-	if (ensure_dir_exists (dest_dir)) {
+	if (ensure_dir_exists (dest_dir, 0700)) {
 		xmlSetDocCompressMode (doc, 3);
 		xmlSaveFile (comment_file, doc);
 	}
@@ -383,10 +383,8 @@ comments_save_comment (const gchar *filename,
 gchar *
 comments_get_comment_as_string (CommentData *data)
 {
-	gchar *s = NULL;
+	gchar *as_string = NULL;
 	gchar time_txt[50];
-	gchar *comment_txt;
-	gchar *place_txt;
 	struct tm *tm;
 
 	if (data == NULL) return NULL;
@@ -397,32 +395,37 @@ comments_get_comment_as_string (CommentData *data)
 	} else
 		time_txt[0] = 0;
 
-	if (data->comment)
-		comment_txt = g_strdup (data->comment);
-	else
-		comment_txt = g_strdup ("");
-
-	if (data->place)
-		place_txt = g_strdup (data->place);
-	else
-		place_txt = g_strdup ("");
-
-	if ((data->comment == NULL) && (data->place == NULL) 
+	if ((data->comment == NULL) 
+	    && (data->place == NULL) 
 	    && (data->time == 0)) {
 		if (data->keywords_n > 0) 
-			s = NULL;
+			as_string = NULL;
 		else
-			s = g_strdup (_("(No Comment)"));
-	} else
-		s = g_strconcat (comment_txt,
-				 (data->comment && data->place) ? ", " : "",
-				 place_txt, 
-				 (data->comment || data->place) ? " " : "",
-				 time_txt, NULL);
+			as_string = g_strdup (_("(No Comment)"));
+	} else {
+		GString *comment;
 
-	g_free (comment_txt);
-	g_free (place_txt);
+		comment = g_string_new ("");
 
-	return s;
+		if (data->comment != NULL)
+			g_string_append (comment, data->comment);
+
+		if (data->place != NULL) {
+			if (data->comment != NULL)
+				g_string_append (comment, ", ");
+			g_string_append (comment, data->place);
+		}
+
+		if (*time_txt != 0) {
+			if ((data->comment != NULL) || (data->place != NULL))
+				g_string_append_c (comment, ' ');
+			g_string_append (comment, time_txt);
+		}
+
+		as_string = comment->str;
+		g_string_free (comment, FALSE);
+	}
+
+	return as_string;
 }
 
