@@ -625,7 +625,7 @@ file_copy (const gchar *from,
 		return FALSE;
 
 	dest_dir = remove_level_from_path (to);
-	if (! ensure_dir_exists (dest_dir)) {
+	if (! ensure_dir_exists (dest_dir, 0755)) {
 		g_free (dest_dir);
 		fclose (fin);
 		return FALSE;
@@ -849,7 +849,8 @@ remove_ending_separator (const gchar *path)
 
 
 gboolean 
-ensure_dir_exists (const gchar *a_path)
+ensure_dir_exists (const gchar *a_path, 
+		   mode_t mode)
 {
 	if (! a_path) return FALSE;
 
@@ -868,7 +869,7 @@ ensure_dir_exists (const gchar *a_path)
 				}
 
 			if (! path_is_dir (path)) {
-					if (mkdir (path, 0755) < 0) {
+					if (mkdir (path, mode) < 0) {
 						g_warning ("directory creation failed: %s.", path);
 						g_free (path);
 						return FALSE;
@@ -912,3 +913,63 @@ dir_list_filter_and_sort (GList *dir_list,
 
 	return filtered;
 }
+
+
+/* characters to escape */
+static gchar bad_char[] = { '$', '\'', '`', '"', '\\', '!', '?', 
+			    ' ', '(', ')', '[', ']', '&', '|' };
+
+/* the size of bad_char */
+static const gint bad_chars = sizeof (bad_char) / sizeof (gchar);
+
+
+/* counts how many characters to escape in @str. */
+static gint
+count_chars_to_escape (const gchar *str)
+{
+	const gchar *s;
+	gint i, n;
+
+	n = 0;
+	for (s = str; *s != 0; s++)
+		for (i = 0; i < bad_chars; i++)
+			if (*s == bad_char[i]) {
+				n++;
+				break;
+			}
+	return n;
+}
+
+
+/* escape with backslash the file name. */
+gchar*
+shell_escape (const gchar *filename)
+{
+	gchar *escaped;
+	gint i, new_l;
+	const gchar *s;
+	gchar *t;
+
+	if (filename == NULL) 
+		return NULL;
+
+	new_l = strlen (filename) + count_chars_to_escape (filename);
+	escaped = g_malloc (new_l + 1);
+
+	s = filename;
+	t = escaped;
+	while (*s) {
+		gboolean is_bad;
+	
+		is_bad = FALSE;
+		for (i = 0; (i < bad_chars) && !is_bad; i++)
+			is_bad = (*s == bad_char[i]);
+
+		if (is_bad)
+			*t++ = '\\';
+		*t++ = *s++;
+	}
+	*t = 0;
+
+	return escaped;
+}
