@@ -23,6 +23,8 @@
 #ifndef FILE_UTILS_H
 #define FILE_UTILS_H
 
+#include <sys/stat.h>
+#include <sys/types.h>
 #include <time.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include "typedefs.h"
@@ -71,7 +73,8 @@ gboolean            file_move                    (const gchar *from,
 
 gint                file_in_path                 (const gchar *name);
 
-gboolean            ensure_dir_exists            (const gchar *a_path);
+gboolean            ensure_dir_exists            (const gchar *a_path,
+						  mode_t mode);
 
 gboolean            file_is_hidden               (const gchar *name);
 
@@ -126,4 +129,7 @@ GList *                 dir_list_filter_and_sort   (GList *dir_list,
 						    gboolean names_only,
 						    gboolean show_dot_files);
 
+gchar*                  shell_escape               (const gchar *filename);
+
+
 #endif /* FILE_UTILS_H */
