@@ -31,19 +31,19 @@
 
 #include "image-list.h"
 
+
 #define DEFAULT_MEDIUM_FONT \
-          "-adobe-helvetica-bold-r-normal-*-*-100-*-*-p-*-iso8859-1,"  \
-          "-*-*-bold-r-normal-*-*-100-*-*-*-*-*-*,*"
+	"-*-*-bold-r-normal--*-120-*-*-*-*-iso8859-1,*"
 
 #define DEFAULT_SMALL_FONT \
-          "-adobe-helvetica-medium-r-normal-*-*-100-*-*-p-*-iso8859-1,"  \
-          "-*-*-medium-r-normal-*-*-100-*-*-*-*-*-*,*"
+	"-*-*-medium-r-normal--*-110-*-*-*-*-iso8859-1,*"
+
 
 /* default spacings */
 #define DEFAULT_ROW_SPACING   4
-#define DEFAULT_COL_SPACING   2
+#define DEFAULT_COL_SPACING   4
 #define DEFAULT_TEXT_SPACING  2
-#define DEFAULT_IMAGE_BORDER  2
+#define DEFAULT_IMAGE_BORDER  3
 #define TEXT_COMMENT_SPACE    5  /* space between text and comment. */
 
 /* Autoscroll timeout in milliseconds */
@@ -141,6 +141,9 @@ typedef struct {
 	/* Rubberband rectangle */
 	GnomeCanvasItem *sel_rect;
 
+	/* focus rectangle */
+	GnomeCanvasItem *focus_rect;
+
 	/* Saved event for a pending selection */
 	GdkEvent select_pending_event;
 
@@ -233,7 +236,6 @@ int
 image_list_get_images_per_line (ImageList *gil)
 {
 	g_return_val_if_fail (IS_IMAGE_LIST (gil), 1);
-
 	return gil_get_images_per_line (gil);
 }
 
@@ -267,7 +269,6 @@ gil_place_image (ImageList *gil,
 	y = y + image_height + priv->text_spacing;
 
 	if (view_comment) {
-
 		gnome_canvas_item_show (GNOME_CANVAS_ITEM (image->comment));
 		gnome_icon_text_item_setxy (image->comment,
 					    x + 1,
@@ -563,6 +564,15 @@ gil_scrollbar_adjust (ImageList *gil)
 	gil->adj->value = wy;
 
 	gtk_adjustment_changed (gil->adj);
+
+	/**/
+	gnome_canvas_item_set (priv->focus_rect,
+			       "x1", 0.0,
+			       "x2", (double) GTK_WIDGET (gil)->allocation.width - 1.0,
+			       "y1", 0.0,
+			       "y2", (double) MAX (height, GTK_WIDGET (gil)->allocation.height) - 1.0,
+						   
+			       NULL);
 }
 
 
@@ -716,9 +726,8 @@ selection_one_image_event (ImageList *gil,
 		priv->edit_pending = FALSE;
 		priv->select_pending = FALSE;
 
-		/* Ignore wheel mouse clicks for now */
 		if (event->button.button > 3)
-			break;
+			return FALSE;
 
 		if (!image->selected) {
 			image_list_unselect_all (gil, NULL, NULL);
@@ -879,10 +888,9 @@ selection_many_image_event (ImageList *gil,
 		priv->edit_pending = FALSE;
 		priv->select_pending = FALSE;
 
-		/* Ignore wheel mouse clicks for now */
 		if (event->button.button > 3)
-			break;
-
+			return FALSE;
+                
 		do_select = TRUE;
 
 		if (additive || range) {
@@ -1193,6 +1201,10 @@ image_new_from_pixbuf (ImageList *gil,
 	gtk_signal_connect (GTK_OBJECT (image->text), "height_changed",
 			    GTK_SIGNAL_FUNC (height_changed),
 			    image);
+
+	/**/
+	gnome_canvas_item_raise_to_top (priv->focus_rect);
+
 	return image;
 }
 
@@ -1412,10 +1424,9 @@ image_list_remove (ImageList *gil, gint pos)
 
 	if (! priv->frozen) {
 		/* FIXME: Optimize, only re-layout from pos to end */
+		/*gil_layout_all_images (gil);*/
 		
-		gil_layout_from_line (gil, pos / gil_get_images_per_line (gil));
-
-		gil_layout_all_images (gil);
+		gil_layout_from_line (gil, pos / gil_get_images_per_line(gil));
 		gil_scrollbar_adjust (gil);
 	} else
 		priv->dirty = TRUE;
@@ -1745,8 +1756,8 @@ gil_button_press (GtkWidget *widget,
 	gil = IMAGE_LIST (widget);
 	priv = gil->priv;
 
-	gtk_widget_grab_focus (widget);
-
+	if (! GTK_WIDGET_HAS_FOCUS (widget))
+		gtk_widget_grab_focus (widget);
 
 	/* ---- FIXME ---- */
 
@@ -1757,6 +1768,24 @@ gil_button_press (GtkWidget *widget,
 			return TRUE;
 */
 
+	/* Mouse wheel support */
+
+	if ((event->type == GDK_BUTTON_PRESS)
+	    && ((event->button == 4) || 
+		(event->button == 5))) {
+		gint value;
+		
+		value = (gil->adj->value 
+			 + (event->button == 4 ? -1 : 1)
+			 * gil->adj->page_increment / 3);
+		value = CLAMP (value, 
+			       gil->adj->lower, 
+			       gil->adj->upper - gil->adj->page_size);
+		gtk_adjustment_set_value (gil->adj, value);
+		
+		return TRUE;
+	}
+
 	pos = image_list_get_image_at (gil, event->x, event->y);
 	if (pos != -1) {
 		ImageListPrivate *priv = gil->priv;
@@ -1898,6 +1927,10 @@ image_is_in_area (ImageList *gil,
 }
 
 
+static int get_first_visible_at_offset (ImageList *gil, gdouble ofs);
+static int get_last_visible_at_offset (ImageList *gil, gdouble ofs);
+
+
 /* Updates the rubberband selection to the specified point */
 static void
 update_drag_selection (ImageList *gil, 
@@ -1905,11 +1938,13 @@ update_drag_selection (ImageList *gil,
 		       gint y)
 {
 	ImageListPrivate *priv;
-	gint x1, x2, y1, y2;
-	GList *l, *begin, *end;
-	gint i, begin_idx;
-	Image *image;
-	gint additive, invert;
+	gint    x1, x2, y1, y2;
+	GList  *l, *begin, *end;
+	gint    i, begin_idx, end_idx;
+	Image  *image;
+	gint    additive, invert;
+	gint    min_y, max_y;
+        double  old_y1_d, old_y2_d;
 
 	priv = gil->priv;
 
@@ -1943,6 +1978,13 @@ update_drag_selection (ImageList *gil,
 	if (y2 >= priv->total_height)
 		y2 = priv->total_height - 1;
 
+	gtk_object_get (GTK_OBJECT (priv->sel_rect),
+			"y1", &old_y1_d,
+			"y2", &old_y2_d,
+			NULL);
+        min_y = MIN (y1, (int) old_y1_d);
+        max_y = MAX (y2, (int) old_y2_d);
+
 	gnome_canvas_item_set (priv->sel_rect,
 			       "x1", (double) x1,
 			       "y1", (double) y1,
@@ -1955,11 +1997,13 @@ update_drag_selection (ImageList *gil,
 	additive = priv->sel_state & GDK_SHIFT_MASK;
 	invert = priv->sel_state & GDK_CONTROL_MASK;
 
-	/* Consider only visible images. */
+	/* Consider only images in the min_y --> max_y offset. */
+
+        begin_idx = get_first_visible_at_offset (gil, min_y);
+        begin = g_list_nth (priv->image_list, begin_idx);
+        end_idx = get_last_visible_at_offset (gil, max_y);
+        end = g_list_nth (priv->image_list, end_idx);
 
-	begin_idx = image_list_get_first_visible (gil);
-	begin = g_list_nth (priv->image_list, begin_idx);
-	end = g_list_nth (priv->image_list, image_list_get_last_visible (gil));
 	if (end != NULL)
 		end = end->next;
 
@@ -2095,6 +2139,28 @@ gil_motion_notify (GtkWidget *widget, GdkEventMotion *event)
 }
 
 
+static gint
+gil_focus_in (GtkWidget *widget,
+	      GdkEventFocus *event)
+{
+	ImageList *gil = IMAGE_LIST (widget);
+	ImageListPrivate *priv = gil->priv;
+	gnome_canvas_item_show (GNOME_CANVAS_ITEM (priv->focus_rect));
+	return FALSE;
+}
+
+
+static gint
+gil_focus_out (GtkWidget *widget,
+	       GdkEventFocus *event)
+{
+	ImageList *gil = IMAGE_LIST (widget);
+	ImageListPrivate *priv = gil->priv;
+	gnome_canvas_item_hide (GNOME_CANVAS_ITEM (priv->focus_rect));
+	return FALSE;
+}
+
+
 static void
 gil_set_scroll_adjustments (GtkLayout *layout,
 			    GtkAdjustment *hadjustment,
@@ -2253,6 +2319,9 @@ gil_class_init (ImageListClass *gil_class)
 	widget_class->button_release_event = gil_button_release;
 	widget_class->motion_notify_event = gil_motion_notify;
 
+	widget_class->focus_in_event = gil_focus_in;
+	widget_class->focus_out_event = gil_focus_out;
+
 	/* we override GtkLayout's set_scroll_adjustments signal instead
 	 * of creating a new signal so as to keep binary compatibility.
 	 * Anyway, a widget class only needs one of these signals, and
@@ -2302,6 +2371,17 @@ gil_init (ImageList *gil)
 	 * adjustments */
 	gnome_canvas_set_scroll_region (GNOME_CANVAS (gil), 0.0, 0.0, 1000000.0, 1000000.0);
 	gnome_canvas_scroll_to (GNOME_CANVAS (gil), 0, 0);
+
+	/* create the focus rectangle. */
+	priv->focus_rect = gnome_canvas_item_new (gnome_canvas_root (GNOME_CANVAS (gil)),
+						  gnome_canvas_rect_get_type (),
+						  "x1", 1.0,
+						  "y1", 1.0,
+						  "x2", 50.0,
+						  "y2", 50.0,
+						  "outline_color", "black",
+						  "width_pixels", 1,
+						  NULL);
 }
 
 
@@ -2369,7 +2449,12 @@ image_list_set_image_width (ImageList *gil, int w)
 static void
 gil_adj_value_changed (GtkAdjustment *adj, 
 		       ImageList *gil)
-{
+{	
+	ImageListPrivate *priv;
+
+	g_return_if_fail (IS_IMAGE_LIST (gil));
+	priv = gil->priv;
+
 	gnome_canvas_scroll_to (GNOME_CANVAS (gil), 0, adj->value);
 }
 
@@ -2932,6 +3017,7 @@ image_list_moveto (ImageList *gil,
 	ImageLine *il;
 	GList *l;
 	gint i, y, uh, line;
+	gfloat value;
 
 	g_return_if_fail (IS_IMAGE_LIST (gil));
 	g_return_if_fail (pos >= 0 && pos < gil->images);
@@ -2953,7 +3039,10 @@ image_list_moveto (ImageList *gil,
 	il = l->data;
 
 	uh = GTK_WIDGET (gil)->allocation.height - image_line_height (gil,il);
-	gtk_adjustment_set_value (gil->adj, y - uh * yalign);
+	value = CLAMP (y - uh * yalign, 
+		       0.0, 
+		       gil->adj->upper - gil->adj->page_size);
+	gtk_adjustment_set_value (gil->adj, value);
 }
 
 
@@ -3276,7 +3365,6 @@ image_list_get_old_text (ImageList *gil)
 	ImageListPrivate *priv;
 
 	g_return_val_if_fail (IS_IMAGE_LIST (gil), NULL);
-
 	priv = gil->priv;
 
 	return priv->old_text;
@@ -3289,76 +3377,84 @@ image_list_get_list (ImageList *gil)
 	ImageListPrivate *priv;
 
 	g_return_val_if_fail (IS_IMAGE_LIST (gil), NULL);
-
 	priv = gil->priv;
 	
 	return priv->image_list;
 }
 
 
-int
-image_list_get_first_visible (ImageList *gil)
+static int
+get_first_visible_at_offset (ImageList *gil, gdouble ofs)
 {
-	gint pos, line;
 	ImageListPrivate *priv;
-	gdouble ofs;
-	GList *l;
+        gint pos, line;
+        GList *l;
 
-	g_return_val_if_fail (IS_IMAGE_LIST (gil), -1);
-
-	if (gil->images == 0)
-		return -1;
+        if (gil->images == 0)
+                return -1;
 
 	priv = gil->priv;
 
-	line = 0;
-	ofs = gil->adj->value;
-	for (l = priv->lines; l && (ofs > 0.0); l = l->next) {
-		ofs -= image_line_height (gil, l->data);
-		line++;
-	}
-	
-	pos = image_list_get_images_per_line (gil) * (line - 1);
+        line = 0;
+        for (l = priv->lines; l && (ofs > 0.0); l = l->next) {
+                ofs -= image_line_height (gil, l->data);
+                line++;
+        }
+        
+        pos = image_list_get_images_per_line (gil) * (line - 1);
 
-	if (pos < 0)
-		pos = 0;
-	if (pos >= gil->images)
-		pos = gil->images - 1;
+        if (pos < 0)
+                pos = 0;
+        if (pos >= gil->images)
+                pos = gil->images - 1;
 
-	return pos;
+        return pos;
+}
+
+
+int
+image_list_get_first_visible (ImageList *gil)
+{
+        g_return_val_if_fail (IS_IMAGE_LIST (gil), -1);
+        return get_first_visible_at_offset (gil, gil->adj->value);
+}
+
+
+static int
+get_last_visible_at_offset (ImageList *gil, gdouble ofs)
+{
+	ImageListPrivate *priv;
+        gint pos, line;
+        GList *l;
+
+        if (gil->images == 0)
+                return -1;
+
+	priv = gil->priv;
+
+        line = 0;
+        for (l = priv->lines; l && (ofs > 0.0); l = l->next) {
+                ofs -= image_line_height (gil, l->data);
+                line++;
+        }
+        
+        pos = image_list_get_images_per_line (gil) * line - 1;
+
+        if (pos < 0)
+                pos = 0;
+        if (pos >= gil->images)
+                pos = gil->images - 1;
+
+        return pos;
 }
 
 
 int
 image_list_get_last_visible (ImageList *gil)
 {
-	gint pos, line;
-	ImageListPrivate *priv;
-	gdouble ofs;
-	GList *l;
-
-	g_return_val_if_fail (IS_IMAGE_LIST (gil), -1);
-
-	if (gil->images == 0)
-		return -1;
-
-	priv = gil->priv;
-
-	line = 0;
-	ofs = gil->adj->value + gil->adj->page_size;
-	for (l = priv->lines; l && (ofs > 0.0); l = l->next) {
-		ofs -= image_line_height (gil, l->data);
-		line++;
-	}
-	
-	pos = image_list_get_images_per_line (gil) * line - 1;
-
-	if (pos < 0)
-		pos = 0;
-	if (pos >= gil->images)
-		pos = gil->images - 1;
-
-	return pos;
+        g_return_val_if_fail (IS_IMAGE_LIST (gil), -1);
+        return get_last_visible_at_offset (gil, (gil->adj->value 
+                                                 + gil->adj->page_size));
 }
 
 
