@@ -47,6 +47,10 @@ preferences_init ()
 	preferences.wallpaper = gnome_config_get_string ("/Background/Default/wallpaper");
 	preferences.wallpapers_dir = gnome_config_get_string ("/Background/Default/wallpapers_dir");
 	preferences.wallpaperAlign = gnome_config_get_int ("/Background/Default/wallpaperAlign");
+
+	preferences.editors_n = 0;
+	preferences.editors = NULL;
+
 	png_exporter_pref.text_font = NULL;
 }
 
@@ -67,6 +71,9 @@ void preferences_release ()
 	if (preferences.wallpapers_dir)
 		g_free (preferences.wallpapers_dir);
 
+	g_list_foreach (preferences.editors, (GFunc) g_free, NULL);
+	g_list_free (preferences.editors);
+
 	if (png_exporter_pref.text_font != NULL)
 		g_free (png_exporter_pref.text_font);
 }
@@ -117,13 +124,13 @@ preferences_set_default_values ()
 	preferences.ss_direction = DIRECTION_FORWARD;
 	preferences.ss_delay = 4;
 	preferences.ss_wrap_around = FALSE;
+	preferences.ss_fullscreen = FALSE;
 
 	preferences.file_list_sort = SORT_BY_NAME;
 	preferences.file_list_sort_type = GTK_SORT_ASCENDING;
 
 	/* Visualization options. */
 	preferences.layout_type = 0;
-	preferences.toolbar_style = TOOLBAR_STYLE_ICONS;
 	preferences.location_on_main_bar = TRUE;
 	preferences.main_win_width = 680;
 	preferences.main_win_height = 500;
@@ -173,6 +180,8 @@ get_bool_with_default (gchar *config_path, gboolean def)
 void 
 preferences_load () 
 {
+	int i;
+
 	/* Startup dir. */
 	preferences.go_to_last_location = gnome_config_get_bool_with_default (
 		"gthumb/Start Up/use last location", FALSE);
@@ -243,6 +252,8 @@ preferences_load ()
 		"gthumb/Slide Show/delay", 4);
 	preferences.ss_wrap_around = get_bool_with_default (
 		"gthumb/Slide Show/wrap around", FALSE);
+	preferences.ss_fullscreen = get_bool_with_default (
+		"gthumb/Slide Show/fullscreen", FALSE);
 
 	/**/
 	preferences.file_list_sort = get_int_with_default (
@@ -255,8 +266,6 @@ preferences_load ()
 		"gthumb/Main Window/location on main bar", TRUE);
 	preferences.layout_type = gnome_config_get_int (
 		"gthumb/Main Window/layout=1");
-	preferences.toolbar_style = get_int_with_default (
-		"gthumb/Main Window/toolbar style", TOOLBAR_STYLE_ICONS);
 	preferences.main_win_width = gnome_config_get_int (
 		"gthumb/Main Window/width=680");
 	preferences.main_win_height = gnome_config_get_int (
@@ -266,6 +275,19 @@ preferences_load ()
 	preferences.sidebar_content_height = gnome_config_get_int (
 		"gthumb/Sidebar/content height=150");
 
+	preferences.editors_n = gnome_config_get_int (
+		"gthumb/Editors/editors=0");
+	for (i = 0; i < preferences.editors_n; i++) {
+		gchar *prop_name;
+		gchar *prop_val;
+
+		prop_name = g_strdup_printf ("gthumb/Editors/editor%d", i);
+		prop_val = gnome_config_get_string (prop_name);
+		if (prop_val != NULL)
+			preferences.editors = g_list_prepend (preferences.editors, prop_val);
+		g_free (prop_name);
+	}
+
 	/* Exporter preferences. */
 
 	/* * Page */
@@ -331,6 +353,9 @@ preferences_load ()
 void 
 preferences_save ()
 {
+	GList *scan;
+	int i;
+
 	/* Startup dir. */
 	gnome_config_set_bool ("gthumb/Start Up/use last location",
 			       preferences.go_to_last_location);
@@ -387,6 +412,8 @@ preferences_save ()
 			       preferences.ss_delay);
 	gnome_config_set_bool ("gthumb/Slide Show/wrap around",
 			       preferences.ss_wrap_around);
+	gnome_config_set_bool ("gthumb/Slide Show/fullscreen",
+			       preferences.ss_fullscreen);
 
 	/**/
 	gnome_config_set_int  ("gthumb/File List/Sort",
@@ -399,8 +426,6 @@ preferences_save ()
 			       preferences.location_on_main_bar);
 	gnome_config_set_int  ("gthumb/Main Window/layout",
 			       preferences.layout_type);
-	gnome_config_set_int  ("gthumb/Main Window/toolbar style",
-			       preferences.toolbar_style);
 	gnome_config_set_int  ("gthumb/Main Window/width",
 			       preferences.main_win_width);
 	gnome_config_set_int  ("gthumb/Main Window/height",
@@ -410,6 +435,13 @@ preferences_save ()
 	gnome_config_set_int  ("gthumb/Sidebar/content height",
 			       preferences.sidebar_content_height);
 
+	gnome_config_set_int ("gthumb/Editors/editors", preferences.editors_n);
+	for (i = 0, scan = preferences.editors; scan; scan = scan->next) {
+		gchar *prop_name;
+		prop_name = g_strdup_printf ("gthumb/Editors/editor%d", i++);
+		gnome_config_set_string (prop_name, (gchar*) scan->data);
+		g_free (prop_name);
+	}
 
 	/* Exporter preferences. */
 
