@@ -41,8 +41,9 @@
 /* If the file size is greater than this the thumbnail will not be
  * created, for functionality reasons. **/
 #define MAX_FILE_SIZE (4*1024*1024)
-#define CACHE_MAX_W   85
-#define CACHE_MAX_H   85
+
+#define CACHE_MAX_W   128
+#define CACHE_MAX_H   128
 
 
 typedef struct
@@ -82,12 +83,12 @@ static void         thumb_loader_done_cb    (ImageLoader *il,
 static void         thumb_loader_error_cb   (ImageLoader *il,
 					     gpointer data);
 
-static gint         normalize_thumb         (gint *width, 
+static gint         scale_thumb             (gint *width, 
 					     gint *height, 
 					     gint max_w, 
 					     gint max_h);
 
-static gint         scale_thumb             (gint *width, 
+static gint         normalize_thumb         (gint *width, 
 					     gint *height, 
 					     gint max_w, 
 					     gint max_h);
@@ -365,7 +366,7 @@ thumb_loader_start (ThumbLoader *tl)
 
 		if (path_is_file (cache_path)) {
 			if (get_file_mtime (cache_path) < get_file_mtime (path)) {
-				/* The thumbnail is old. */
+				/* Old mtime : recreate thumbnail. */
 				g_free (cache_path);
 				cache_path = NULL;
 			}
@@ -437,7 +438,7 @@ thumb_from_xpm_d (const char **data,
 	w = gdk_pixbuf_get_width (pixbuf);
 	h = gdk_pixbuf_get_height (pixbuf);
 
-	if (normalize_thumb (&w, &h, max_w, max_h)) {
+	if (scale_thumb (&w, &h, max_w, max_h)) {
 		/* Scale */
 		GdkPixbuf *tmp;
 		
@@ -479,9 +480,11 @@ thumb_loader_save_to_cache (ThumbLoader *tl)
 	if (cache_dir == NULL)
 		return FALSE;
 
-	if (ensure_dir_exists (cache_dir))
+	if (ensure_dir_exists (cache_dir, 0700))
 		success = pixbuf_to_file_as_png (priv->pixbuf, cache_path);
 
+	/**/
+
 	g_free (cache_dir);
 	g_free (cache_path);
 
@@ -496,6 +499,7 @@ thumb_loader_done_cb (ImageLoader *il,
 	ThumbLoader *tl = data;
 	ThumbLoaderPrivateData *priv = tl->priv;
 	GdkPixbuf *pixbuf;
+	GdkInterpType itype;
 	gint width, height;
 	gboolean modified;
 
@@ -513,33 +517,55 @@ thumb_loader_done_cb (ImageLoader *il,
 	width = gdk_pixbuf_get_width (pixbuf);
 	height = gdk_pixbuf_get_height (pixbuf);
 
-	/* Thumbnails are always saved with the same size, then scaled if
-	 * necessary. */
+	itype = (preferences.thumb_quality == ZOOM_QUALITY_HIGH) 
+		? GDK_INTERP_BILINEAR 
+		: GDK_INTERP_NEAREST;
 
-	/* Check whether to scale. */
-	modified = normalize_thumb (&width, &height, CACHE_MAX_W, CACHE_MAX_H);
-	if (modified) {
-		gdk_pixbuf_unref (priv->pixbuf);
-		priv->pixbuf = gdk_pixbuf_scale_simple (pixbuf, width, height, 
-							(preferences.thumb_quality == ZOOM_QUALITY_HIGH) ? GDK_INTERP_BILINEAR : GDK_INTERP_NEAREST);
-	}
+	if (priv->use_cache) {
+		/* Thumbnails are always saved with the same size, then 
+		 * scaled if necessary. */
 
-	/* Save the thumbnail if necessary. */
-	if (! priv->from_cache
-	    && priv->use_cache
-	    && preferences.enable_thumb_caching 
-	    && modified) 
-		thumb_loader_save_to_cache (tl);
-       
-
-	/* Scale if the user wants a different size. */
-	modified = scale_thumb (&width, &height, priv->max_w, priv->max_h);
-	if (modified) {
-		pixbuf = priv->pixbuf;
-		priv->pixbuf = gdk_pixbuf_scale_simple (pixbuf, 
-							width, height, 
-							(preferences.thumb_quality == ZOOM_QUALITY_HIGH) ? GDK_INTERP_BILINEAR : GDK_INTERP_NEAREST);
-		gdk_pixbuf_unref (pixbuf);
+		/* Check whether to scale. */
+		modified = scale_thumb (&width, &height, 
+					CACHE_MAX_W, CACHE_MAX_H);
+		if (modified) {
+			gdk_pixbuf_unref (priv->pixbuf);
+			priv->pixbuf = gdk_pixbuf_scale_simple (pixbuf, 
+								width, 
+								height,
+								itype);
+		}
+		
+		/* Save the thumbnail if necessary. */
+		if (! priv->from_cache
+		    && preferences.enable_thumb_caching)
+			thumb_loader_save_to_cache (tl);
+		
+		/* Scale if the user wants a different size. */
+		modified = normalize_thumb (&width, 
+					    &height,
+					    priv->max_w, 
+					    priv->max_h);
+		if (modified) {
+			pixbuf = priv->pixbuf;
+			priv->pixbuf = gdk_pixbuf_scale_simple (pixbuf, 
+								width, 
+								height, 
+								itype);
+			gdk_pixbuf_unref (pixbuf);
+		}
+	} else {
+		modified = scale_thumb (&width, 
+					&height, 
+					priv->max_w, 
+					priv->max_h);
+		if (modified) {
+			gdk_pixbuf_unref (priv->pixbuf);
+			priv->pixbuf = gdk_pixbuf_scale_simple (pixbuf, 
+								width,
+								height,
+								itype);
+		}
 	}
 
 	gtk_signal_emit (GTK_OBJECT (tl), thumb_loader_signals[DONE]);
@@ -568,10 +594,10 @@ thumb_loader_error_cb (ImageLoader *il,
 
 
 static gint 
-normalize_thumb (gint *width, 
-		 gint *height, 
-		 gint max_width, 
-		 gint max_height)
+scale_thumb (gint *width, 
+	     gint *height, 
+	     gint max_width, 
+	     gint max_height)
 {
 	gboolean modified;
 	gfloat max_w = max_width;
@@ -581,7 +607,8 @@ normalize_thumb (gint *width,
 	gfloat factor;
 	gint new_width, new_height;
 
-	if ((*width <= max_width) && (*height <= max_height)) return FALSE;
+	if ((*width < max_width - 1) && (*height < max_height - 1)) 
+		return FALSE;
 
 	factor = MIN (max_w / w, max_h / h);
 	new_width  = MAX ((gint) (w * factor), 1);
@@ -597,10 +624,10 @@ normalize_thumb (gint *width,
 
 
 static gint 
-scale_thumb (gint *width, 
-	     gint *height, 
-	     gint max_width, 
-	     gint max_height)
+normalize_thumb (gint *width, 
+		 gint *height, 
+		 gint max_width, 
+		 gint max_height)
 {
 	gboolean modified;
 	gfloat max_w = max_width;
@@ -610,7 +637,10 @@ scale_thumb (gint *width,
 	gfloat factor;
 	gint new_width, new_height;
 
-	if ((*width < CACHE_MAX_W) && (*height < CACHE_MAX_H)) 
+	if ((max_width > CACHE_MAX_W) && (max_height > CACHE_MAX_H)) {
+		if ((*width < CACHE_MAX_W - 1) && (*height < CACHE_MAX_H - 1)) 
+			return FALSE;
+	} else if ((*width < max_width - 1) && (*height < max_height - 1)) 
 		return FALSE;
 
 	factor = MIN (max_w / w, max_h / h);
