@@ -40,7 +40,10 @@ typedef struct {
 
 	GtkWidget *dialog;
 	GtkWidget *app_list_clist;
-	GtkWidget *cancel_btn;
+	GtkWidget *recent_clist;
+	GtkWidget *app_entry;
+	GtkWidget *del_button;
+
 	GList     *app_list;
 	GList     *file_list;
 } DialogData;
@@ -60,89 +63,69 @@ open_with__destroy_cb (GtkWidget *widget,
 }
 
 
-/* counts how many characters of type @c are in @str. */
-static gint
-count (const gchar *str, char c, gint str_len)
+static void
+exec_command (gchar *application, GList *file_list)
 {
-	const gchar *s;
-	gint n;
+        GString * command;
+        GList *   scan;
 
-	for (n = 0, s = str; s != str + str_len; s++)
-		if (*s == c) n++;
+        command = g_string_new ("");
+        g_string_append (command, application);
+        for (scan = file_list; scan; scan = scan->next) {
+                gchar *filename = scan->data;
+                gchar *e_filename;
 
-	return n;
-}
+                g_string_append_c (command, ' ');
+                e_filename = shell_escape (filename);
+                g_string_append (command, e_filename);
 
+                g_free (e_filename);
+        }
 
-/* escape with backslash the file name. */
-static gchar*
-shell_escape (const gchar *filename)
-{
-	static gchar bad_char[] = {'$', '\'', '`', '"', '\\', '!', '?', ' ', 
-				   '(', ')', '[', ']'};
-	static const gint bad_chars = sizeof (bad_char) / sizeof (gchar);
-	gchar *escaped;
-	gint i, l, new_l;
-	const gchar *s;
-	gchar *t;
-
-	if (filename == NULL) return NULL;
-
-	new_l = l = strlen (filename);
-	for (i = 0; i < bad_chars; i++)
-		new_l += count (filename, bad_char[i], l);
-	escaped = g_malloc (new_l + 1);
-
-	s = filename;
-	t = escaped;
-	while (*s) {
-		gboolean is_bad;
-	
-		is_bad = FALSE;
-		for (i = 0; (i < bad_chars) && !is_bad; i++)
-			is_bad = (*s == bad_char[i]);
-
-		if (is_bad)
-			*t++ = '\\';
-		*t++ = *s++;
-	}
-	*t = 0;
-
-	return escaped;
+        gnome_execute_shell (g_get_home_dir (), command->str); 
+        g_string_free (command, TRUE);
 }
 
 
 static void
-exec_command (gchar *application, GList *file_list)
+open_cb (GtkWidget *widget,
+	 gpointer callback_data)
 {
+	DialogData *data = callback_data;
+	gchar *application;
 	GList *scan;
-	gchar *files;
-	gint files_length;
-	gchar *command;
+	gboolean present = FALSE;
 
-	/* compute the file list length. */
-	files_length = 0;
-	for (scan = file_list; scan; scan = scan->next)
-		files_length += strlen (scan->data) + 1;
+	application = gtk_entry_get_text (GTK_ENTRY (data->app_entry));
 
-	files = g_malloc (files_length + 1);
-	*files = 0;
-	for (scan = file_list; scan; scan = scan->next) {
-		gchar *name_escaped;
+	/* add the command to the editors list if not already present. */
 
-		if (scan != file_list)
-			files = strcat (files, " ");
-
-		name_escaped = shell_escape (scan->data);
-		files = strncat (files, name_escaped, strlen (name_escaped));
-		g_free (name_escaped);
+	for (scan = data->app_list; scan; scan = scan->next) {
+		GnomeVFSMimeApplication *app = scan->data;
+		if (strcmp (app->command, application) == 0) {
+			present = TRUE;
+			break;
+		}
 	}
 
-	command = g_strdup_printf ("%s %s", application, files);
-	g_free (files);
+	for (scan = preferences.editors; scan; scan = scan->next) {
+		gchar *command = scan->data;
+		if (strcmp (command, application) == 0) {
+			present = TRUE;
+			break;
+		}
+	}
 
-        gnome_execute_shell (g_get_home_dir (), command);
-	g_free (command);
+	if (! present) {
+		preferences.editors_n++;
+		preferences.editors = g_list_prepend (preferences.editors,
+						      g_strdup (application));
+	}
+
+	/* exec the application. */
+
+	exec_command (application, data->file_list);
+	gtk_widget_destroy (data->dialog);
 }
 
 
@@ -156,9 +139,46 @@ app_selected_cb (GtkWidget *widget,
 	DialogData *data = callback_data;
 	GnomeVFSMimeApplication *app;
 	
-	app = gtk_clist_get_row_data (GTK_CLIST (widget), row);
-	exec_command (app->command, data->file_list);
-	gtk_widget_destroy (data->dialog);
+	app = gtk_clist_get_row_data (GTK_CLIST (data->app_list_clist), row);
+	gtk_entry_set_text (GTK_ENTRY (data->app_entry), app->command);
+}
+
+
+static void 
+recent_selected_cb (GtkWidget *widget, 
+		    gint row, 
+		    gint col,
+		    GdkEvent *event,
+		    gpointer callback_data)
+{
+	DialogData *data = callback_data;
+	gchar *editor;
+
+	editor = gtk_clist_get_row_data (GTK_CLIST (data->recent_clist), row);
+	gtk_entry_set_text (GTK_ENTRY (data->app_entry), editor);
+}
+
+
+static void
+delete_recent_cb (GtkWidget *widget,
+		  gpointer callback_data)
+{
+	DialogData *data = callback_data;
+	GList *selection;
+	gint row;
+	gchar *editor;
+
+	selection = GTK_CLIST (data->recent_clist)->selection;
+	if (selection == NULL)
+		return;
+
+	row = GPOINTER_TO_INT (selection->data);
+	editor = gtk_clist_get_row_data (GTK_CLIST (data->recent_clist), row);
+
+	gtk_clist_remove (GTK_CLIST (data->recent_clist), row);
+
+	preferences.editors = g_list_remove (preferences.editors, editor);
+	g_free (editor);
 }
 
 
@@ -175,6 +195,7 @@ open_with_cb (GtkWidget *widget,
 	GnomeVFSMimeApplication *app;
 	gint row;
 	GList *app_names = NULL;
+	GtkWidget *ok_btn, *cancel_btn;
 
 	data = g_new (DialogData, 1);
 
@@ -184,7 +205,7 @@ open_with_cb (GtkWidget *widget,
 		/* no selection. */
 		
 		if ((window->image_path == NULL) 
-		    || !path_is_file (window->image_path)) {
+		    || ! path_is_file (window->image_path)) {
 			/* no image in the viewer. */
 			g_free (data);
 			return;
@@ -204,22 +225,47 @@ open_with_cb (GtkWidget *widget,
 
 	data->dialog = glade_xml_get_widget (data->gui, "open_with_dialog");
 	data->app_list_clist = glade_xml_get_widget (data->gui, "app_list_clist");
-	data->cancel_btn = glade_xml_get_widget (data->gui, "open_with_cancel_btn");
+	data->recent_clist = glade_xml_get_widget (data->gui, "recent_clist");
+	ok_btn = glade_xml_get_widget (data->gui, "open_with_ok_btn");
+	cancel_btn = glade_xml_get_widget (data->gui, "open_with_cancel_btn");
+	data->app_entry = glade_xml_get_widget (data->gui, "open_with_app_entry");
+	data->del_button = glade_xml_get_widget (data->gui, "delete_button");
 
 	/* Set the signals handlers. */
 
-	gtk_signal_connect (GTK_OBJECT (data->dialog), "destroy",
+	gtk_signal_connect (GTK_OBJECT (data->dialog), 
+			    "destroy",
 			    (GtkSignalFunc) open_with__destroy_cb,
 			    data);
 	gtk_signal_connect (GTK_OBJECT (data->app_list_clist), 
 			    "select_row",
 			    (GtkSignalFunc) app_selected_cb, 
 			    data);
-	gtk_signal_connect_object (GTK_OBJECT (data->cancel_btn), "clicked",
+	gtk_signal_connect (GTK_OBJECT (data->recent_clist), 
+			    "select_row",
+			    (GtkSignalFunc) recent_selected_cb, 
+			    data);
+	gtk_signal_connect_object (GTK_OBJECT (cancel_btn), 
+				   "clicked",
 				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
 				   GTK_OBJECT (data->dialog));
+	gtk_signal_connect (GTK_OBJECT (ok_btn), 
+			    "clicked",
+			    GTK_SIGNAL_FUNC (open_cb),
+			    data);
+	gtk_signal_connect_object (GTK_OBJECT (cancel_btn), 
+				   "clicked",
+				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
+				   GTK_OBJECT (data->dialog));
+	gtk_signal_connect (GTK_OBJECT (data->del_button), 
+			    "clicked",
+			    GTK_SIGNAL_FUNC (delete_recent_cb),
+			    data);
+
 	/* Set data. */
 
+	/* * registered editors list. */
+
 	data->app_list = NULL;
 	for (scan = data->file_list; scan; scan = scan->next) {
 		const char *result;
@@ -267,6 +313,24 @@ open_with_cb (GtkWidget *widget,
 	gtk_clist_sort (GTK_CLIST (data->app_list_clist));
 
 
+	/* * recent editors list. */
+
+	gtk_clist_column_titles_passive (GTK_CLIST (data->recent_clist));
+	gtk_clist_set_sort_column (GTK_CLIST (data->recent_clist), 1);
+
+	col_data[1] = NULL;
+	for (scan = preferences.editors; scan; scan = scan->next) {
+		gchar *editor = scan->data;
+
+		col_data[0] = editor;
+		row = gtk_clist_append (GTK_CLIST (data->recent_clist), 
+					col_data);
+		gtk_clist_set_row_data (GTK_CLIST (data->recent_clist), 
+					row,
+					editor);
+	}
+	gtk_clist_sort (GTK_CLIST (data->recent_clist));
+
 	/* Run dialog. */
 	gtk_window_set_transient_for (GTK_WINDOW (data->dialog), 
 				      GTK_WINDOW (window->app));
