@@ -46,7 +46,7 @@
 #include "icons/layout3.xpm"
 #include "icons/layout4.xpm"
 
-static gint thumb_size[] = {48, 64, 85, 95, 112, 128};
+static gint thumb_size[] = {48, 64, 85, 95, 112, 128, 164, 200, 256};
 static gint thumb_sizes = sizeof (thumb_size) / sizeof (gint);
 
 typedef struct {
@@ -68,7 +68,6 @@ typedef struct {
 	GtkWidget *radio_layout3;
 	GtkWidget *radio_layout4;
 	GtkWidget *toggle_location_on_main_bar;
-	GtkWidget *opt_toolbar_style;
 
 	GtkWidget *radio_from_extension;
 	GtkWidget *radio_from_content;
@@ -91,9 +90,11 @@ typedef struct {
 	GtkWidget *opt_ss_direction;
 	GtkWidget *spin_ss_delay;
 	GtkWidget *toggle_ss_wrap_around;
+	GtkWidget *toggle_ss_fullscreen;
 } DialogData;
 
 
+static gint     original_layout_type;
 static guint    original_zoom_quality;
 static guint    original_zoom_change;
 TranspType      original_transp_type;
@@ -141,6 +142,7 @@ static void
 save_original_values (GThumbWindow *window)
 {
 	/* Save original values. */
+	original_layout_type = preferences.layout_type;
 	original_zoom_quality = preferences.zoom_quality;
 	original_zoom_change = preferences.zoom_change;
 	original_transp_type = image_viewer_get_transp_type (IMAGE_VIEWER (window->viewer));
@@ -206,6 +208,7 @@ apply_cb (GtkWidget *widget,
 
 	/* Slide Show. */
 	preferences.ss_wrap_around = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->toggle_ss_wrap_around));
+	preferences.ss_fullscreen = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->toggle_ss_fullscreen));
 	preferences.ss_delay = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (data->spin_ss_delay));
 	preferences.ss_direction = opt_menu_get_active_idx (data->opt_ss_direction);
 
@@ -232,10 +235,11 @@ apply_cb (GtkWidget *widget,
 	else
 		preferences.layout_type = 3;
 
-	preferences.toolbar_style = opt_menu_get_active_idx (data->opt_toolbar_style);
-
 	preferences.location_on_main_bar = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (data->toggle_location_on_main_bar));
 
+	if (original_layout_type != preferences.layout_type)
+		window_notify_update_layout (data->window);
+
 	save_original_values (data->window);
 	gnome_property_box_changed (GNOME_PROPERTY_BOX (data->dialog));
 }
@@ -322,6 +326,7 @@ dlg_preferences (GThumbWindow *window)
 	data->gui = glade_xml_new (GTHUMB_GLADEDIR "/" GLADE_FILE , NULL);
         if (!data->gui) {
                 g_warning ("Could not find " GLADE_FILE "\n");
+		g_free (data);
                 return;
         }
 
@@ -343,7 +348,6 @@ dlg_preferences (GThumbWindow *window)
 	data->radio_layout2 = glade_xml_get_widget (data->gui, "radio_layout2");
 	data->radio_layout3 = glade_xml_get_widget (data->gui, "radio_layout3");
 	data->radio_layout4 = glade_xml_get_widget (data->gui, "radio_layout4");
-	data->opt_toolbar_style = glade_xml_get_widget (data->gui, "opt_toolbar_style");
 	data->toggle_location_on_main_bar = glade_xml_get_widget (data->gui, "toggle_location_on_main_bar");
 
 
@@ -368,6 +372,7 @@ dlg_preferences (GThumbWindow *window)
         data->opt_ss_direction = glade_xml_get_widget (data->gui, "opt_ss_direction");
         data->spin_ss_delay = glade_xml_get_widget (data->gui, "spin_ss_delay");
         data->toggle_ss_wrap_around = glade_xml_get_widget (data->gui, "toggle_ss_wrap_around");
+        data->toggle_ss_fullscreen = glade_xml_get_widget (data->gui, "toggle_ss_fullscreen");
 
         btn_ok = GNOME_PROPERTY_BOX (data->dialog)->ok_button;
         btn_cancel = GNOME_PROPERTY_BOX (data->dialog)->cancel_button;
@@ -413,7 +418,6 @@ dlg_preferences (GThumbWindow *window)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->radio_layout3), TRUE);
 	else
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->radio_layout4), TRUE);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (data->opt_toolbar_style), preferences.toolbar_style);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->toggle_location_on_main_bar), preferences.location_on_main_bar);
 
 	/* * browser */
@@ -447,6 +451,7 @@ dlg_preferences (GThumbWindow *window)
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (data->spin_ss_delay),
 				   (gfloat) preferences.ss_delay);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->toggle_ss_wrap_around), preferences.ss_wrap_around);
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (data->toggle_ss_fullscreen), preferences.ss_fullscreen);
 
 	/* Set the signals handlers. */
 
