@@ -410,6 +410,21 @@ cancel_progress_dlg_cb (GtkWidget *widget,
 }
 
 
+/* called when the "help" button in the search dialog is pressed. */
+static void
+help_cb (GtkWidget *widget, 
+	 DialogData *data)
+{
+	gchar *tmp;
+
+	tmp = gnome_help_file_find_file ("gthumb", "find.html");
+	if (tmp) {
+		gnome_help_goto (0, tmp);
+		g_free (tmp);
+	}
+}
+
+
 static void
 dlg_search_ui (GThumbWindow *window, 
 	       gchar *catalog_path, 
@@ -418,6 +433,7 @@ dlg_search_ui (GThumbWindow *window,
 	DialogData *data;
 	GtkWidget *btn_search;
 	GtkWidget *btn_close;
+	GtkWidget *btn_help;
 
 	data = g_new (DialogData, 1);
 
@@ -469,6 +485,7 @@ dlg_search_ui (GThumbWindow *window,
 
         btn_search = glade_xml_get_widget (data->gui, "s_search_button");
         btn_close = glade_xml_get_widget (data->gui, "s_close_button");
+	btn_help = glade_xml_get_widget (data->gui, "s_help_button");
 
 	/* Set widgets data. */
 
@@ -515,6 +532,9 @@ dlg_search_ui (GThumbWindow *window,
 	gtk_signal_connect_object (GTK_OBJECT (btn_close), "clicked",
 				   GTK_SIGNAL_FUNC (gtk_widget_destroy),
 				   GTK_OBJECT (data->dialog));
+	gtk_signal_connect (GTK_OBJECT (btn_help), "clicked",
+			    GTK_SIGNAL_FUNC (help_cb),
+			    data);
 
 	gtk_signal_connect (GTK_OBJECT (data->search_progress_dialog), "destroy",
 			    (GtkSignalFunc) destroy_progress_cb,
