@@ -97,14 +97,14 @@ ensure_directories_exist ()
 			    "/",
 			    RC_THUMBS_DIR,
 			    NULL);
-	ensure_dir_exists (path);
+	ensure_dir_exists (path, 0700);
 	g_free (path);
 
 	path = g_strconcat (g_get_home_dir (),
 			    "/",
 			    RC_CATALOG_DIR,
 			    NULL);
-	ensure_dir_exists (path);
+	ensure_dir_exists (path, 0700);
 	g_free (path);
 }
 
