@@ -169,7 +169,7 @@ GnomeUIInfo edit_menu[] = {
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
 
-	GNOMEUIINFO_SUBTREE (N_("Edit Catalog"), edit_catalog_menu),
+	GNOMEUIINFO_SUBTREE (N_("_Edit Catalog"), edit_catalog_menu),
 
 	GNOMEUIINFO_SEPARATOR,
 
@@ -184,13 +184,13 @@ GnomeUIInfo edit_menu[] = {
 	GNOMEUIINFO_SUBTREE (N_("A_lter Image"), alter_image_menu),
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("_Edit Comment..."), N_("Add a comment to selected images"), 
+	  N_("Ed_it Comment..."), N_("Add a comment to selected images"), 
 	  dlg_edit_comment, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 
 	  0, 0, NULL },
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("_Delete Comment"), N_("Delete comments from selected images"),
+	  N_("Dele_te Comment"), N_("Delete comments from selected images"),
 	  delete_comment_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
@@ -204,7 +204,7 @@ GnomeUIInfo edit_menu[] = {
 	  0, 0, NULL },
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("R_emove from Catalog"), N_("Remove selected files from the catalog"), 
+	  N_("Remo_ve from Catalog"), N_("Remove selected files from the catalog"), 
 	  remove_from_catalog_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
@@ -295,13 +295,13 @@ GnomeUIInfo trans_type_menu[] = {
 
 GnomeUIInfo zoom_quality_radio_list[] = {
 	{ GNOME_APP_UI_ITEM, 
-	  N_("High Quality"), " ", 
+	  N_("_High Quality"), " ", 
 	  high_quality_zoom_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("Low Quality"), " ", 
+	  N_("_Low Quality"), " ", 
 	  low_quality_zoom_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
@@ -312,25 +312,25 @@ GnomeUIInfo zoom_quality_radio_list[] = {
 
 GnomeUIInfo zoom_menu[] = {
 	{ GNOME_APP_UI_ITEM, 
-	  N_("Zoom In"), " ", 
+	  N_("Zoom _In"), " ", 
 	  zoom_in_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  '+', 0, NULL },
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("Zoom Out"), " ", 
+	  N_("Zoom _Out"), " ", 
 	  zoom_out_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  '-', 0, NULL },
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("Actual Size"), " ", 
+	  N_("_Actual Size"), " ", 
 	  no_zoom_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  '1', 0, NULL },
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("Zoom to Fit"), N_("Zoom to fit window"), 
+	  N_("Zoom to _Fit"), N_("Zoom to fit window"), 
 	  zoom_to_fit_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  'x', GDK_CONTROL_MASK, NULL },
@@ -420,7 +420,7 @@ GnomeUIInfo go_menu[] = {
 	  'u', GDK_CONTROL_MASK, NULL },
 
 	{ GNOME_APP_UI_ITEM,
-	  N_("Go to Container Directory"), " ",
+	  N_("_Go to Container Directory"), " ",
 	  go_to_container_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_JUMP_TO,
 	  0, 0, NULL },
@@ -488,13 +488,13 @@ GnomeUIInfo tools_menu[] = {
 	GNOMEUIINFO_SEPARATOR,
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("_Find images..."), " ", 
+	  N_("_Find Images..."), " ", 
 	  dlg_search, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SEARCH,
 	  'f', GDK_CONTROL_MASK, NULL },
 
 	{ GNOME_APP_UI_ITEM,
-	  N_("Sa_ve Selection as Image..."), " ",
+	  N_("Create _Index Image..."), " ",
 	  export_as_image_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_SAVE_AS,
 	  0, 0, NULL },
@@ -502,13 +502,13 @@ GnomeUIInfo tools_menu[] = {
 	GNOMEUIINFO_SEPARATOR,
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("_Remove Old Thumbnails"), N_("Remove old thumbnails from the cache"), 
+	  N_("Remove Old _Thumbnails"), N_("Remove old thumbnails from the cache"), 
 	  remove_old_thumbnails_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("_Remove Old Comments"), N_("Remove old comments from the cache"), 
+	  N_("Remove Old _Comments"), N_("Remove old comments from the cache"), 
 	  remove_old_comments_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
@@ -578,7 +578,32 @@ GnomeUIInfo main_menu[] = {
 
 /* popup menus. */
 
+enum {
+	FILE_POPUP_MENU_VIEW_FULLSCREEN = 0,
+	FILE_POPUP_MENU_OPEN_WITH,
+	FILE_POPUP_MENU_SEP1,
+	FILE_POPUP_MENU_RENAME_FILE,
+	FILE_POPUP_MENU_DELETE_FILES,
+	FILE_POPUP_MENU_COPY_FILES,
+	FILE_POPUP_MENU_MOVE_FILES,
+	FILE_POPUP_MENU_SEP2,
+	FILE_POPUP_MENU_EDIT_COMMENT,
+	FILE_POPUP_MENU_DELETE_COMMENT,
+	FILE_POPUP_MENU_SEP3,
+	FILE_POPUP_MENU_ADD_TO_CAT,
+	FILE_POPUP_MENU_REMOVE_FROM_CAT,
+	FILE_POPUP_MENU_GO_TO_CONTAINER_DIR,
+	FILE_POPUP_MENU_END
+};
+
+
 GnomeUIInfo file_popup_menu_data[] = {
+	{ GNOME_APP_UI_ITEM,
+	  N_("_View Full Screen"), " ", 
+          view_fullscreen_cb, NULL, NULL,
+          GNOME_APP_PIXMAP_NONE, 0,
+          0, 0, NULL },
+
 	{ GNOME_APP_UI_ITEM, 
 	  N_("_Open with..."), N_("Open selected files with an application"), 
 	  open_with_cb, NULL, NULL,
@@ -614,13 +639,13 @@ GnomeUIInfo file_popup_menu_data[] = {
 	GNOMEUIINFO_SEPARATOR,
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("_Edit Comment..."), N_("Add a comment to selected images"), 
+	  N_("Ed_it Comment..."), N_("Add a comment to selected images"), 
 	  dlg_edit_comment, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_PROP, 
 	  0, 0, NULL },
 
 	{ GNOME_APP_UI_ITEM, 
-	  N_("_Delete Comment"), N_("Delete comments from selected images"),
+	  N_("Dele_te Comment"), N_("Delete comments from selected images"),
 	  delete_comment_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
@@ -640,7 +665,7 @@ GnomeUIInfo file_popup_menu_data[] = {
 	  0, 0, NULL },
 
 	{ GNOME_APP_UI_ITEM,
-	  N_("Go to Container Directory"), " ",
+	  N_("_Go to Container Directory"), " ",
 	  go_to_container_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_JUMP_TO,
 	  0, 0, NULL },
@@ -688,25 +713,25 @@ GnomeUIInfo catalog_popup_menu_data[] = {
 
 GnomeUIInfo image_popup_menu_data[] = {
 	{ GNOME_APP_UI_ITEM,
-	  N_("Zoom In"), " ",
+	  N_("Zoom _In"), " ",
 	  zoom_in_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
 
 	{ GNOME_APP_UI_ITEM,
-	  N_("Zoom Out"), " ",
+	  N_("Zoom _Out"), " ",
 	  zoom_out_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
 
 	{ GNOME_APP_UI_ITEM,
-	  N_("Actual Size"), " ",
+	  N_("_Actual Size"), " ",
 	  no_zoom_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
 
 	{ GNOME_APP_UI_ITEM,
-	  N_("Zoom to Fit"), " ",
+	  N_("Zoom to _Fit"), " ",
 	  zoom_to_fit_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
@@ -714,7 +739,7 @@ GnomeUIInfo image_popup_menu_data[] = {
 	GNOMEUIINFO_SEPARATOR,
 
 	{ GNOME_APP_UI_TOGGLEITEM, 
-	  N_("Full Screen"), " ", 
+	  N_("_Full Screen"), " ", 
 	  fullscreen_cb, NULL, NULL,
 	  GNOME_APP_PIXMAP_NONE, 0,
 	  0, 0, NULL },
