@@ -28,6 +28,9 @@
 #include <gtk/gtkwidget.h>
 
 
+void    view_fullscreen_cb       (GtkWidget *widget,
+                                  void *data);
+
 void    add_to_catalog_cb        (GtkWidget *widget,
 				  void *data);
 
