@@ -28,6 +28,7 @@
 #include <libgnomeui/gnome-window-icon.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
+#include <libgnomevfs/gnome-vfs-result.h>
 
 #include "auto-completion.h"
 #include "thumb-cache.h"
@@ -102,26 +103,18 @@ static void
 view_image_at_pos (GThumbWindow *window, 
 		   gint pos)
 {
-	gchar *path;
 	ImageList *ilist = IMAGE_LIST (window->file_list->ilist);
-	
-	/* If the image is from a catalog remember the catalog name. */
-	if (window->image_catalog != NULL) {
-		g_free (window->image_catalog);
-		window->image_catalog = NULL;
-	}
+	gchar *path;
 
-	if (window->sidebar_content == CATALOG_LIST)
-		window->image_catalog = g_strdup (window->catalog_path);
-	
 	/* Load the image. */
 	path = file_list_path_from_row (window->file_list, pos);
 	window_load_image (window, path);
 	g_free (path);
 	
 	/* Center row if not fully visible. */
+
 	if (image_list_image_is_visible (ilist, pos) != GTK_VISIBILITY_FULL)
-		image_list_moveto (ilist, pos, 0.5); 
+		image_list_moveto (ilist, pos, 0.5);
 }
 
 
@@ -154,19 +147,19 @@ file_button_press_cb (GtkWidget *widget,
 
 	/* Button 3 pressed. */
 
-	gtk_widget_set_sensitive (window->popupmenu_file[2],
+	gtk_widget_set_sensitive (window->popupmenu_file[FILE_POPUP_MENU_RENAME_FILE],
 				  ilist_utils_only_one_is_selected (ilist));
-	gtk_widget_set_sensitive (window->popupmenu_file[12],
+	gtk_widget_set_sensitive (window->popupmenu_file[FILE_POPUP_MENU_GO_TO_CONTAINER_DIR],
 				  ilist_utils_only_one_is_selected (ilist));
 
 	if (window->sidebar_content == CATALOG_LIST) {
-		HIDE (window->popupmenu_file[10]);
-		SHOW (window->popupmenu_file[11]);
-		SHOW (window->popupmenu_file[12]);
+		HIDE (window->popupmenu_file[FILE_POPUP_MENU_ADD_TO_CAT]);
+		SHOW (window->popupmenu_file[FILE_POPUP_MENU_REMOVE_FROM_CAT]);
+		SHOW (window->popupmenu_file[FILE_POPUP_MENU_GO_TO_CONTAINER_DIR]);
 	} else {
-		SHOW (window->popupmenu_file[10]);
-		HIDE (window->popupmenu_file[11]);
-		HIDE (window->popupmenu_file[12]);
+		SHOW (window->popupmenu_file[FILE_POPUP_MENU_ADD_TO_CAT]);
+		HIDE (window->popupmenu_file[FILE_POPUP_MENU_REMOVE_FROM_CAT]);
+		HIDE (window->popupmenu_file[FILE_POPUP_MENU_GO_TO_CONTAINER_DIR]);
 	}
 
 	gtk_menu_popup (GTK_MENU (window->file_popup_menu),
@@ -677,10 +670,9 @@ key_press_cb (GtkWidget *widget,
 	case GDK_b:
 	case GDK_BackSpace:
 	case GDK_Up:
-	case GDK_Page_Up:
+
 	case GDK_Left: 
 	case GDK_KP_Up:
-	case GDK_KP_Page_Up:
 	case GDK_KP_Left:
 		window_show_prev_image (window);
 		break;
@@ -689,10 +681,9 @@ key_press_cb (GtkWidget *widget,
 	case GDK_space:
 	case GDK_Right:
 	case GDK_Down:
-	case GDK_Page_Down: 
+
 	case GDK_KP_Right:
 	case GDK_KP_Down:
-	case GDK_KP_Page_Down:
 		window_show_next_image (window);
 		break;
 
@@ -717,6 +708,54 @@ key_press_cb (GtkWidget *widget,
 			remove_from_catalog_cb (NULL, window);
 		break;
 
+		/* Page Down */
+	case GDK_Page_Down:
+	case GDK_KP_Page_Down:
+		{
+			ImageList *ilist;
+			gint pos;
+
+			ilist = IMAGE_LIST (window->file_list->ilist);
+			pos = image_list_get_last_visible (ilist);
+			pos += image_list_get_images_per_line (ilist);
+			pos = CLAMP (pos, 0, ilist->images - 1);
+			file_list_select_image_by_row (window->file_list, pos);
+			view_image_at_pos (window, pos);
+		}
+		break;
+
+		/* Page Up */
+	case GDK_Page_Up:
+	case GDK_KP_Page_Up:
+		{
+			ImageList *ilist;
+			gint pos;
+
+			ilist = IMAGE_LIST (window->file_list->ilist);
+			pos = image_list_get_first_visible (ilist);
+			pos -= image_list_get_images_per_line (ilist);
+			pos = CLAMP (pos, 0, ilist->images - 1);
+			file_list_select_image_by_row (window->file_list, pos);
+			view_image_at_pos (window, pos);
+		}
+		break;
+
+		/* Rename */
+	case GDK_F2:
+		{
+			ImageList *ilist;
+			gint pos;
+
+			if (window->image_path == NULL)
+				break;
+
+			ilist = IMAGE_LIST (window->file_list->ilist);
+			pos = file_list_row_from_path (window->file_list, 
+						       window->image_path);
+			image_list_start_editing (ilist, pos);
+		}
+		break;
+
 	default:
 		break;
 	}
@@ -1203,9 +1242,7 @@ file_list_text_changed (ImageList *ilist,
 		g_free (msg);
 
 		result = FALSE;
-	}
-
-	if (! rename (old_filepath, new_filepath)) {
+	} else if (! rename (old_filepath, new_filepath)) {
 		cache_move (old_filepath, new_filepath);
 		all_windows_notify_file_rename (old_filepath, new_filepath);
 	} else {
@@ -1256,7 +1293,6 @@ window_new (gboolean view_first_image)
 	GtkWidget    *toolbar;
 	gint          e_width;
 	gint          i;
-	GtkToolbarStyle toolbar_style;
 
 	window = g_new (GThumbWindow, 1);
 
@@ -1276,17 +1312,18 @@ window_new (gboolean view_first_image)
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	/* Create the widgets. */
+
 	if ((preferences.layout_type == 0) || (preferences.layout_type == 2))
-		window->paned = paned1 = gedo_hpaned_new (); 
+		window->main_pane = paned1 = gedo_hpaned_new (); 
 	else
-		window->paned = paned1 = gedo_vpaned_new (); 
+		window->main_pane = paned1 = gedo_vpaned_new (); 
 	gedo_paned_child1_use_minsize (GEDO_PANED (paned1), TRUE, 80);
 	gnome_app_set_contents (GNOME_APP (window->app), paned1);
 
 	if ((preferences.layout_type == 0) || (preferences.layout_type == 2))
-		window->content_paned = paned2 = gedo_vpaned_new (); 
+		window->content_pane = paned2 = gedo_vpaned_new (); 
 	else
-		window->content_paned = paned2 = gedo_hpaned_new (); 
+		window->content_pane = paned2 = gedo_hpaned_new (); 
 	gedo_paned_child1_use_minsize (GEDO_PANED (paned2), TRUE, 80);
 	gedo_paned_child2_use_minsize (GEDO_PANED (paned2), TRUE, 80);
 	gedo_paned_add1 (GEDO_PANED (paned1), paned2);
@@ -1379,7 +1416,7 @@ window_new (gboolean view_first_image)
 	gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
 	gtk_container_add (GTK_CONTAINER (frame), window->viewer);
 
-	table = gtk_table_new (2, 2, FALSE);
+	window->image_table = table = gtk_table_new (2, 2, FALSE);
 	vscrollbar = gtk_vscrollbar_new (IMAGE_VIEWER (window->viewer)->vadj);
 	hscrollbar = gtk_hscrollbar_new (IMAGE_VIEWER (window->viewer)->hadj);
 	event_box = gtk_event_box_new ();
@@ -1470,25 +1507,16 @@ window_new (gboolean view_first_image)
 				       0,
 				       window);
 
-	/* Create the toolbar. */
-	switch (preferences.toolbar_style) {
-	case TOOLBAR_STYLE_DEFAULT:
-		toolbar_style = GTK_TOOLBAR_BOTH;
-		break;
-
-	case TOOLBAR_STYLE_ICONS:
-	default:
-		toolbar_style = GTK_TOOLBAR_ICONS;
-		break;
-	}
 	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, 
-				   toolbar_style);
+				   GTK_TOOLBAR_ICONS);
+	window->toolbar = toolbar;
 	gnome_app_fill_toolbar_with_data (GTK_TOOLBAR (toolbar), 
 					  toolbar_data, 
 					  (GtkAccelGroup*) NULL,
 					  window);
 
 	/* Add a location entry to the toolbar. */
+	window->location_on_main_bar = preferences.location_on_main_bar;
 	window->location_entry = gtk_entry_new ();
 	gtk_signal_connect_after (GTK_OBJECT (window->location_entry),
 				  "key_press_event",
@@ -1559,7 +1587,7 @@ window_new (gboolean view_first_image)
 	window->image_info = gtk_label_new (" ");
 	gtk_misc_set_alignment (GTK_MISC (window->image_info), 0.5, 0.5);
 	gtk_widget_set_usize (window->image_info, 
-                              gdk_string_width (gtk_widget_get_style (window->image_info)->font, "MMMMMMMMMMMMMMMMMMMMMMMM"), -1);
+                              gdk_string_width (gtk_widget_get_style (window->image_info)->font, "MMMMMMMMMMMMMMMMMMMMMMMMM"), -1);
 	gtk_container_add (GTK_CONTAINER (frame), window->image_info);
         gtk_box_pack_start (GTK_BOX (window->statusbar), frame, 
 			    FALSE, TRUE, 0);
@@ -1615,7 +1643,7 @@ window_new (gboolean view_first_image)
 	window->mitem_export = tools_menu[4].widget;
 
 	window->popmitem_fullscreen = image_popup_menu_data[5].widget;
-	for (i = 0; i < 13; i++) 
+	for (i = 0; i < FILE_POPUP_MENU_END; i++) 
 		window->popupmenu_file[i] = file_popup_menu_data[i].widget;
 	for (i = 0; i < 6; i++) 
 		window->popupmenu_catalog[i] = catalog_popup_menu_data[i].widget;
@@ -1654,6 +1682,8 @@ window_new (gboolean view_first_image)
 
 	window->view_first_image = view_first_image;
 
+	window->view_image_timer = 0;
+
 	/* Update menu items status. */
 
 	set_check_menu_item_state (window, window->mitem_thumbnails, 
@@ -1759,8 +1789,8 @@ window_close (GThumbWindow *window)
 	/* Save visualization options. */
 
 	if (window->sidebar_visible) {
-		preferences.sidebar_width = gedo_paned_get_position (GEDO_PANED (window->paned));
-		preferences.sidebar_content_height = gedo_paned_get_position (GEDO_PANED (window->content_paned));
+		preferences.sidebar_width = gedo_paned_get_position (GEDO_PANED (window->main_pane));
+		preferences.sidebar_content_height = gedo_paned_get_position (GEDO_PANED (window->content_pane));
 	}
 
 	gdk_window_get_size (window->app->window, 
@@ -1948,8 +1978,8 @@ window_hide_sidebar (GThumbWindow *window)
 		return;
 
 	window->sidebar_visible = FALSE;
-	window->sidebar_width = gedo_paned_get_position (GEDO_PANED (window->paned));
-	gedo_paned_hide_child1 (GEDO_PANED (window->paned));
+	window->sidebar_width = gedo_paned_get_position (GEDO_PANED (window->main_pane));
+	gedo_paned_hide_child1 (GEDO_PANED (window->main_pane));
 
 	/* Sync menu and toolbar. */
 	set_check_menu_item_state (window, window->mitem_browser, FALSE);
@@ -1969,7 +1999,7 @@ window_show_sidebar (GThumbWindow *window)
 		return;
 
 	window->sidebar_visible = TRUE;
-	gedo_paned_set_position (GEDO_PANED (window->paned), window->sidebar_width); 
+	gedo_paned_set_position (GEDO_PANED (window->main_pane), window->sidebar_width); 
 
 	/* Sync menu and toolbar. */
 	set_check_menu_item_state (window, window->mitem_browser, TRUE);
@@ -2566,7 +2596,7 @@ window_show_last_image (GThumbWindow *window)
 }
 
 
-gint
+static gint
 slideshow_timeout_cb (gpointer data)
 {
 	GThumbWindow *window = data;
@@ -2611,6 +2641,9 @@ window_start_slideshow (GThumbWindow *window)
 
 	window->slideshow = TRUE;
 
+	if (preferences.ss_fullscreen)
+		fullscreen_start (fullscreen, window);
+
 	/* Check menu item. */
 	set_check_menu_item_state (window, window->mitem_slideshow, TRUE);
 
@@ -2646,12 +2679,40 @@ window_stop_slideshow (GThumbWindow *window)
 }
 
 
+static gint
+view_timeout_cb (gpointer data)
+{
+	GThumbWindow *window = data;
+
+	window->view_image_timer = 0;
+	image_viewer_load_image (IMAGE_VIEWER (window->viewer), 
+				 window->image_path);
+	return FALSE;
+}
+
+
 void
 window_load_image (GThumbWindow *window, 
 		   const gchar *filename)
 {
 	g_return_if_fail (window != NULL);
+
+	if (window->view_image_timer != 0) {
+		gtk_timeout_remove (window->view_image_timer);
+		window->view_image_timer = 0;
+	}
 	
+	/* If the image is from a catalog remember the catalog name. */
+
+	if (window->image_catalog != NULL) {
+		g_free (window->image_catalog);
+		window->image_catalog = NULL;
+	}
+	if (window->sidebar_content == CATALOG_LIST)
+		window->image_catalog = g_strdup (window->catalog_path);
+
+	/**/
+
 	if (window->image_path)
 		g_free (window->image_path);
 	window->image_path = NULL;
@@ -2664,7 +2725,10 @@ window_load_image (GThumbWindow *window,
 	}
 		
 	window->image_path = g_strdup (filename);
-	image_viewer_load_image (IMAGE_VIEWER (window->viewer), filename);
+
+	window->view_image_timer = gtk_timeout_add (110, /* FIXME */
+						    view_timeout_cb, 
+						    window);
 }
 
 
@@ -2672,6 +2736,7 @@ static void
 update_statusbar_list_info (GThumbWindow *window)
 {
 	gchar *info, *size_txt, *sel_size_txt;
+	gchar *total_info, *selected_info;
 	gint tot_n, sel_n;
 	GnomeVFSFileSize tot_size, sel_size;
 	GList *scan;
@@ -2702,14 +2767,31 @@ update_statusbar_list_info (GThumbWindow *window)
 
 	size_txt = gnome_vfs_format_file_size_for_display (tot_size);
 	sel_size_txt = gnome_vfs_format_file_size_for_display (sel_size);
-	info = g_strdup_printf ("%d %s (%s) %s: %d %s (%s)",
-				tot_n,
-				(tot_n == 1) ? _("file") : _("files"),
-				size_txt,
-				_("Selected"),
-				sel_n,
-				(sel_n == 1) ? _("file") : _("files"),
-				sel_size_txt);
+
+	if (tot_n == 0)
+		total_info = g_strdup ("");
+	else if (tot_n == 1)
+		total_info = g_strdup_printf (_("1 file (%s)"),
+					      size_txt);
+	else
+		total_info = g_strdup_printf (_("%d files (%s)"),
+					      tot_n,
+					      size_txt); 
+
+	if (sel_n == 0)
+		selected_info = g_strdup ("");
+	else if (sel_n == 1)
+		selected_info = g_strdup_printf (_("1 file selected (%s)"), 
+						 sel_size_txt);
+	else
+		selected_info = g_strdup_printf (_("%d files selected (%s)"), 
+						 sel_n, 
+						 sel_size_txt);
+
+	info = g_strconcat (total_info, 
+			    ((sel_n == 0) ? NULL : ", "),
+			    selected_info, 
+			    NULL);
 
 	gnome_appbar_set_default (GNOME_APPBAR (window->statusbar), info);
 
@@ -3056,6 +3138,56 @@ window_notify_update_directory (GThumbWindow *window,
 }
 
 
+void
+window_notify_update_layout (GThumbWindow *window)
+{
+	GtkWidget    *paned1;      /* Main paned widget. */
+	GtkWidget    *paned2;      /* Secondary paned widget. */
+	gint          paned1_pos;
+	gint          paned2_pos;
+
+	paned1_pos = gedo_paned_get_position (GEDO_PANED (window->main_pane));
+	paned2_pos = gedo_paned_get_position (GEDO_PANED (window->content_pane));
+
+	if ((preferences.layout_type == 0) || (preferences.layout_type == 2))
+		paned1 = gedo_hpaned_new (); 
+	else
+		paned1 = gedo_vpaned_new (); 
+	gedo_paned_child1_use_minsize (GEDO_PANED (paned1), TRUE, 80);
+
+	if ((preferences.layout_type == 0) || (preferences.layout_type == 2))
+		paned2 = gedo_vpaned_new (); 
+	else
+		paned2 = gedo_hpaned_new (); 
+	gedo_paned_child1_use_minsize (GEDO_PANED (paned2), TRUE, 80);
+	gedo_paned_child2_use_minsize (GEDO_PANED (paned2), TRUE, 80);
+	gedo_paned_add1 (GEDO_PANED (paned1), paned2);
+
+	gtk_widget_reparent (window->notebook, paned2);
+	if (preferences.layout_type < 2)
+		gtk_widget_reparent (window->file_list->root_widget, paned2);
+	else
+		gtk_widget_reparent (window->file_list->root_widget, paned1);
+
+	if (preferences.layout_type < 2)
+		gtk_widget_reparent (window->image_table, paned1);
+	else
+		gtk_widget_reparent (window->image_table, paned2);
+
+	gedo_paned_set_position (GEDO_PANED (paned1), paned1_pos);
+	gedo_paned_set_position (GEDO_PANED (paned2), paned2_pos);
+
+	gtk_widget_show (paned1);
+	gtk_widget_show (paned2);
+	gnome_app_set_contents (GNOME_APP (window->app), paned1);
+
+	gtk_widget_destroy (window->main_pane);
+
+	window->main_pane = paned1;
+	window->content_pane = paned2;
+}
+
+
 static gint
 load_progress (gpointer data)
 {
